/*{
    "DESCRIPTION": "fBm Clouds",
    "CREDIT": "patriciogonzalezvivo ported by @colin_movecraft",
    "CATEGORIES": [
        "Noise"
    ],

    "INPUTS": [
    
        {
            "NAME": "scale",
            "TYPE": "float",
            "DEFAULT": 9.0,
            "MIN":0.0,
            "MAX":100.0
        },
        {
            "NAME": "brightness",
            "TYPE": "float",
            "DEFAULT": 0.5,
            "MIN":0.0,
            "MAX":1.0
        },
        {
            "NAME": "evolution",
            "TYPE": "float",
            "DEFAULT": 2.0,
            "MIN":0.0,
            "MAX":10.0
        },
        {
            "NAME": "octaves",
            "TYPE": "float",
            "DEFAULT": 8,
            "MIN":0,
            "MAX":10
        }
        
    ]
}*/



// Author @patriciogv - 2015
// http://patriciogonzalezvivo.com


//random numbers
float random (in vec2 _st) {
    return fract(sin(dot(_st.xy,
                         vec2(12.9898,78.233)))*
        43758.5453123);
}

// Based on Morgan McGuire @morgan3d
// https://www.shadertoy.com/view/4dS3Wd
float noise (in vec2 _st) {
    vec2 i = floor(_st);
    vec2 f = fract(_st);

    // Four corners in 2D of a tile
    float a = random(i);
    float b = random(i + vec2(1.0, 0.0));
    float c = random(i + vec2(0.0, 1.0));
    float d = random(i + vec2(1.0, 1.0));

    vec2 u = f * f * (3.0 - 2.0 * f);

    return mix(a, b, u.x) +
            (c - a)* u.y * (1.0 - u.x) +
            (d - b) * u.x * u.y;
}

int uOctaves = int(octaves);

float fbm ( in vec2 _st) {
    float v = 0.0;
    float a = 0.5;
    vec2 shift = vec2(100.0);
    // Rotate to reduce axial bias
    mat2 rot = mat2(cos(0.5), sin(0.5),
                    -sin(0.5), cos(0.50));
    for (int i = 0; i < uOctaves; ++i) {
        v += a * noise(_st);
        _st = rot * _st * 2.0 + shift;
        a *= 0.5;
    }
    return v;
}

void main() {
    float uScale = scale;
    float uEvolution = evolution;
    
    vec2 st = gl_FragCoord.xy/RENDERSIZE*uScale;
    vec3 color = vec3(0.0);

    vec2 q = vec2(0.);
    q.x = fbm( st + 0.00*(TIME * uEvolution));
    q.y = fbm( st +sin(TIME * .01*(uEvolution)));

    vec2 r = vec2(0.);
    r.x = fbm( st + 1.0*q + vec2(1.7,9.2)+ 0.15*(TIME * uEvolution) );
    r.y = fbm( st + 1.0*q + vec2(8.3,2.8)+ 0.126 * (TIME * uEvolution));

    float f = fbm(st+(r*cos(TIME*uEvolution*.1)));
    
    gl_FragColor = vec4((f*f*f+1.0*f*f+.8*f)*vec3(brightness),1.);
}